@@ -3,18 +3,18 @@
 
 #define SLEPC_VERSION_RELEASE    1
 #define SLEPC_VERSION_MAJOR      3
-#define SLEPC_VERSION_MINOR      12
+#define SLEPC_VERSION_MINOR      13
 #define SLEPC_VERSION_SUBMINOR   2
 #define SLEPC_VERSION_PATCH      0
-#define SLEPC_RELEASE_DATE       "September 30, 2019"
-#define SLEPC_VERSION_DATE       "Jan 13, 2020"
+#define SLEPC_RELEASE_DATE       "March 31, 2020"
+#define SLEPC_VERSION_DATE       "May 12, 2020"
 
 #if !defined (SLEPC_VERSION_GIT)
-#define SLEPC_VERSION_GIT        "v3.12.2"
+#define SLEPC_VERSION_GIT        "v3.13.2"
 #endif
 
 #if !defined(SLEPC_VERSION_DATE_GIT)
-#define SLEPC_VERSION_DATE_GIT   "2020-01-13 12:01:54 +0100"
+#define SLEPC_VERSION_DATE_GIT   "2020-05-12 15:28:21 +0200"
 #endif
 
 #define SLEPC_VERSION_EQ(MAJOR,MINOR,SUBMINOR) \
