@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2019, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -90,6 +90,7 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   eps->rg              = NULL;
   eps->D               = NULL;
   eps->IS              = NULL;
+  eps->ISL             = NULL;
   eps->defl            = NULL;
   eps->eigr            = NULL;
   eps->eigi            = NULL;
@@ -306,6 +307,7 @@ PetscErrorCode EPSDestroy(EPS *eps)
   /* just in case the initial vectors have not been used */
   ierr = SlepcBasisDestroy_Private(&(*eps)->nds,&(*eps)->defl);CHKERRQ(ierr);
   ierr = SlepcBasisDestroy_Private(&(*eps)->nini,&(*eps)->IS);CHKERRQ(ierr);
+  ierr = SlepcBasisDestroy_Private(&(*eps)->ninil,&(*eps)->ISL);CHKERRQ(ierr);
   if ((*eps)->convergeddestroy) {
     ierr = (*(*eps)->convergeddestroy)((*eps)->convergedctx);CHKERRQ(ierr);
   }
