@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2019, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -18,9 +18,9 @@
 
    References:
 
-       [1] C. Campos and J.E. Roman, "A polynomial Jacobi–Davidson solver
-           with support for non-monomial bases and deflation", BIT (in
-           press), 2019.
+       [1] C. Campos and J.E. Roman, "A polynomial Jacobi-Davidson solver
+           with support for non-monomial bases and deflation", BIT Numer.
+           Math. (in press), 2019.
 
        [2] G.L.G. Sleijpen et al., "Jacobi-Davidson type methods for
            generalized eigenproblems and polynomial eigenproblems", BIT
@@ -39,7 +39,7 @@ static PetscBool  cited = PETSC_FALSE;
 static const char citation[] =
   "@Article{slepc-slice-qep,\n"
   "   author = \"C. Campos and J. E. Roman\",\n"
-  "   title = \"A polynomial Jacobi–Davidson solver with support for non-monomial bases and deflation\",\n"
+  "   title = \"A polynomial {Jacobi-Davidson} solver with support for non-monomial bases and deflation\",\n"
   "   journal = \"{BIT} Numer. Math.\",\n"
   "   volume = \"IP\",\n"
   "   number = \"-\",\n"
@@ -290,10 +290,6 @@ static PetscErrorCode PEPJDUpdateTV(PEP pep,PetscInt low,PetscInt high,Vec *w)
 */
 static PetscErrorCode PEPJDOrthogonalize(PetscInt row,PetscInt col,PetscScalar *X,PetscInt ldx,PetscInt *rk,PetscInt *P,PetscScalar *R,PetscInt ldr)
 {
-#if defined(SLEPC_MISSING_LAPACK_GEQP3) || defined(PETSC_MISSING_LAPACK_ORGQR)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GEQP3/QRGQR - Lapack routines are unavailable");
-#else
   PetscErrorCode ierr;
   PetscInt       i,j,n,r;
   PetscBLASInt   row_,col_,ldx_,*p,lwork,info,n_;
@@ -340,7 +336,6 @@ static PetscErrorCode PEPJDOrthogonalize(PetscInt row,PetscInt col,PetscScalar *
   SlepcCheckLapackInfo("orgqr",info);
   ierr = PetscFree4(p,tau,work,rwork);CHKERRQ(ierr);
   PetscFunctionReturn(0);
-#endif
 }
 
 /*
@@ -355,7 +350,7 @@ static PetscErrorCode PEPJDExtendedPCApply(PC pc,Vec x,Vec y)
   PetscErrorCode    ierr;
   const PetscScalar *array1;
   PetscScalar       *x2=NULL,*t=NULL,*ps=NULL,*array2,zero=0.0,sone=1.0;
-  PetscBLASInt      one=1.0,ld_,n_,ncv_;
+  PetscBLASInt      one=1,ld_,n_,ncv_;
   PEP_JD            *pjd=NULL;
 
   PetscFunctionBegin;
@@ -740,7 +735,7 @@ static PetscErrorCode PEPJDProcessInitialSpace(PEP pep,Vec *w)
   PetscFunctionReturn(0);
 }
 
-static PetscErrorCode PEPJDShellMatMult(Mat P,Vec x,Vec y)
+static PetscErrorCode MatMult_PEPJD(Mat P,Vec x,Vec y)
 {
   PetscErrorCode  ierr;
   PEP_JD_MATSHELL *matctx;
@@ -882,7 +877,7 @@ static PetscErrorCode PEPJDShellMatMult(Mat P,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-static PetscErrorCode PEPJDSellMatCreateVecs(Mat A,Vec *right,Vec *left)
+static PetscErrorCode MatCreateVecs_PEPJD(Mat A,Vec *right,Vec *left)
 {
   PetscErrorCode  ierr;
   PEP_JD_MATSHELL *matctx;
@@ -913,10 +908,6 @@ static PetscErrorCode PEPJDSellMatCreateVecs(Mat A,Vec *right,Vec *left)
 
 static PetscErrorCode PEPJDUpdateExtendedPC(PEP pep,PetscScalar theta)
 {
-#if defined(PETSC_MISSING_LAPACK_GESVD) || defined(PETSC_MISSING_LAPACK_GETRI) || defined(PETSC_MISSING_LAPACK_GETRF)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GESVD/GETRI/GETRF - Lapack routines are unavailable");
-#else
   PetscErrorCode ierr;
   PEP_JD         *pjd = (PEP_JD*)pep->data;
   PEP_JD_PCSHELL *pcctx;
@@ -988,7 +979,6 @@ static PetscErrorCode PEPJDUpdateExtendedPC(PEP pep,PetscScalar theta)
     }
   }
   PetscFunctionReturn(0);
-#endif
 }
 
 static PetscErrorCode PEPJDMatSetUp(PEP pep,PetscInt sz,PetscScalar *theta)
@@ -1097,8 +1087,8 @@ static PetscErrorCode PEPJDCreateShellPC(PEP pep,Vec *ww)
   }
   ierr = PetscNew(&matctx);CHKERRQ(ierr);
   ierr = MatCreateShell(PetscObjectComm((PetscObject)pep),kspsf*nloc,kspsf*mloc,PETSC_DETERMINE,PETSC_DETERMINE,matctx,&pjd->Pshell);CHKERRQ(ierr);
-  ierr = MatShellSetOperation(pjd->Pshell,MATOP_MULT,(void(*)(void))PEPJDShellMatMult);CHKERRQ(ierr);
-  ierr = MatShellSetOperation(pjd->Pshell,MATOP_CREATE_VECS,(void(*)(void))PEPJDSellMatCreateVecs);CHKERRQ(ierr);
+  ierr = MatShellSetOperation(pjd->Pshell,MATOP_MULT,(void(*)(void))MatMult_PEPJD);CHKERRQ(ierr);
+  ierr = MatShellSetOperation(pjd->Pshell,MATOP_CREATE_VECS,(void(*)(void))MatCreateVecs_PEPJD);CHKERRQ(ierr);
   matctx->pep = pep;
   target[0] = pep->target; target[1] = 0.0;
   ierr = PEPJDMatSetUp(pep,1,target);CHKERRQ(ierr);
@@ -1130,10 +1120,6 @@ static PetscErrorCode PEPJDCreateShellPC(PEP pep,Vec *ww)
 
 static PetscErrorCode PEPJDEigenvectors(PEP pep)
 {
-#if defined(SLEPC_MISSING_LAPACK_TREVC)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"TREVC - Lapack routine is unavailable");
-#else
   PetscErrorCode ierr;
   PEP_JD         *pjd = (PEP_JD*)pep->data;
   PetscBLASInt   ld,nconv,info,nc;
@@ -1177,7 +1163,6 @@ static PetscErrorCode PEPJDEigenvectors(PEP pep)
   ierr = MatDestroy(&U);CHKERRQ(ierr);
   ierr = PetscFree3(Z,wr,w);CHKERRQ(ierr);
   PetscFunctionReturn(0);
-#endif
 }
 
 static PetscErrorCode PEPJDLockConverged(PEP pep,PetscInt *nv,PetscInt sz)
